library ieee;
use ieee.std_logic_1164.all;

-- Multiplexer with configurable data width

entity mux8 is 
  generic (
    width : integer);
  port (
    d0_i :           in std_ulogic_vector(width-1 downto 0);
    d1_i :           in std_ulogic_vector(width-1 downto 0);
    d2_i :           in std_ulogic_vector(width-1 downto 0);
    d3_i :           in std_ulogic_vector(width-1 downto 0);
    d4_i :           in std_ulogic_vector(width-1 downto 0);
    d5_i :           in std_ulogic_vector(width-1 downto 0);
    d6_i :           in std_ulogic_vector(width-1 downto 0);
    d7_i :           in std_ulogic_vector(width-1 downto 0);
    s_i :            in std_ulogic_vector(2 downto 0); 
    y_o :            out std_ulogic_vector(width-1 downto 0)
  );
end; 

architecture struct of mux8 is

  component mux is 
    generic (
      width : integer);
    port (
      a_i :           in std_ulogic_vector(width-1 downto 0);
      b_i :           in std_ulogic_vector(width-1 downto 0);
      s_i :           in std_ulogic; 
      y_o :           out std_ulogic_vector(width-1 downto 0)
    );
  end component;

  component mux4 is 
    generic (
      width : integer);
    port (
      d0_i :           in std_ulogic_vector(width-1 downto 0);
      d1_i :           in std_ulogic_vector(width-1 downto 0);
      d2_i :           in std_ulogic_vector(width-1 downto 0);
      d3_i :           in std_ulogic_vector(width-1 downto 0);
      s_i :            in std_ulogic_vector(1 downto 0); 
      y_o :            out std_ulogic_vector(width-1 downto 0)
    );
  end component; 
  
  signal s1, s2 : std_ulogic_vector(width-1 downto 0); 
 
begin

  mux_i0 : mux
  generic map (
    width => width)
  port map (
    a_i => s1,
    b_i => s2, 
    s_i => s_i(2), 
    y_o => y_o); 

  mux4_i0 : mux4
  generic map (
    width => width)
  port map (
    d0_i => d0_i,
    d1_i => d1_i,
    d2_i => d2_i, 
    d3_i => d3_i, 
    s_i => s_i(1 downto 0), 
    y_o => s1); 

  mux4_i1 : mux4
  generic map (
    width => width)
  port map (
    d0_i => d4_i,
    d1_i => d5_i,
    d2_i => d6_i, 
    d3_i => d7_i, 
    s_i => s_i(1 downto 0), 
    y_o => s2);   
       
end; -- architecture


